from __future__ import unicode_literals
__pkgname__ = 'opentargets-ontologyutils'
__author__ = 'OpenTargets Core Team <support@targetvalidation.org>'
__author_email__ = 'support@targetvalidation.org'
__license__ = 'Apache License, Version 2.0'
__homepage__ = 'https://github.com/opentargets/ontology-utils'
__version__ = '1.1.0'
__description__ = 'Ontology utilities at targetvalidation.org'
